/**************************************************************
	Low cost 7 channel A/D converter whth LCD display
	 for PIC16F88
 **************************************************************
	author air_variable
	http://www.ne.jp/asahi/air/variable/
	version 1.20
	Tools:
	MPLAB v8.15a
	Hitech PIC C pro9.60pl4
---------------------------------------------------------------
Date	:Time	Version	Build	Note
20081130:2017	1.00	---		Begin the program
20081130:2346	1.00	001		Remarks fix
20081201:0836	1.01	---		Minor fix
20081203:0639	1.10	---		The routine in the bar-graph was changed.
20081203:2050	1.11	---		Minor fix
20081205:2300	1.12	---		additional function LCD display zero suppress
20081206:1120	1.13	---		Fix lcd.c and lcd,h
20090115:2300	1.20	---		Change average algorithm and CG RAM set routine
---------------------------------------------------------------
Project folder(default)
	C:\picsrc\piccpro\lcd_adc
Source Files
	main.c (This file)
	lcd.c
	sci.c
Header Files
	lcd.h
	sci.h	
---------------------------------------------------------------
Pin assignment
	1:RA2:A:analog AN2
	2:RA3:A:analog AN3/VREF+ (not used)
	3:RA4:A:analog AN4
	4:MCLR/RA5:I:Mode switch /PICKIT2-1(W/10kPU)
	5:VSS:GND
	6:RB0:O:LCD DB4
	7:RB1:O:LCD DB5
	8:RB2:I:RX in (w/100kPU)
	9:RB3:O:LCD DB7

	18:RA1:A:analog AN1
	17:RA0:A:analog AN0
	16:RA7:O:LCD E
	15:RA6:O:LCD RS
	14:VDD:Power 5 Volt
	13:RB7:A:AN6 /PICKIT2-5
	12:RB6:A:AN5 /PICKIT2-4
	11:RB5:O:TX out
	10:RB4:O:LCD DB6
**************************************************************/
//CN[ht@CݒƁA__delay֐pgݒ
//U8MHzgp
#include <htc.h>
#include <stdlib.h>
#include "lcd.h"
#include "sci.h"

#define MHz 000000
#define _XTAL_FREQ 8MHz
#define PSW RA5	//switch
#define USART_SPEED	9600	//speed set
#define MAX_BUFF	32		//RX ring buffer size 2^n
#define NEWLINE sci_puts("\r\n") //sACRR[h
#define VERSION_SEND sci_puts("version 1.20\n\r");
#define ADC_response 4	//o[Ot𑜓x(1`4)
//RtBO QƐC:\Program Files\HI-TECH Software\PICC\PRO\9.60\include\pic16f87.h
__CONFIG(
	CCPRB3 & DEBUGDIS & LVPDIS & BOREN & MCLRDIS & PWRTEN & WDTDIS & INTIO
	);
__CONFIG(
	FCMDIS & IESODIS
	);

//EEPROM setting address
//CG RAM user character define start address
#define EEPROM_CGRAM_Start_address		0x00

//CG RAM user character define 8 kinds	//[U[LN^` (2i`s\̂16iŃZbg)
__EEPROM_DATA(0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10);//38-3f (|   )
__EEPROM_DATA(0x14,0x14,0x14,0x14,0x14,0x14,0x14,0x14);//40-47 (||  )
__EEPROM_DATA(0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15);//48-4f (||| )
__EEPROM_DATA(0x00,0x00,0x00,0x00,0x00,0x1f,0x1f,0x00);//50-57 (1 )
__EEPROM_DATA(0x00,0x00,0x00,0x00,0x1f,0x1f,0x1f,0x00);//58-5f (2 )
__EEPROM_DATA(0x00,0x00,0x00,0x1f,0x1f,0x1f,0x1f,0x00);//60-67 (3 )
__EEPROM_DATA(0x00,0x00,0x1f,0x1f,0x1f,0x1f,0x1f,0x00);//68-6f (4 )
__EEPROM_DATA(0x00,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x00);//70-78 (5 )


//O[oݒ
const unsigned char progress_mark[]={
	'.',' '
};

const unsigned char bar_mark[]={//4 level
	' ',0x00,0x01,0x02
};

const unsigned char minibar_mark[]={//7 level
	' ','_',0x03,0x04,0x05,0x06,0x07
};

//RX FIFO Data
//unsigned char Buffer_FIFO[MAX_BUFF];
unsigned char Buffer_FIFO[MAX_BUFF];
unsigned char FIFO_write_pointer=0;
unsigned char FIFO_read_pointer=0;
unsigned char FIFO_datasize=0;
unsigned char FIFO_afterglow_indicate_count=0;//CWP[^[]CJE^

void adinit(void){
	ADON=1;	//ADRo[^dI
	ADFM=1;	//E
	VCFG1=0;	//Vref+ =Vdd
	VCFG0=0;	//Vref- =Vss
	ADCS1 =1;
	ADCS0 =1;	//RCUgp
}

unsigned int adconv(char adselect){	//PIC16F88p
	unsigned int temp;
	ADCON0 = (ADCON0 & 0b11000111) |(( adselect <<3) & 0b00111000);//AD`lZbg 
	__delay_us(20);//20us wait
	ADIF =0;
//	ADIE =1;
//	PEIE =1;
//	GIE  = 0;
	GODONE=1;	//ϊX^[g
//ʐMgpƂ̓X[v@\𖳌ɂ
//	SLEEP();
//	NOP();
	while(GODONE);//ADϊ
//	GIE  = 0;
//	PEIE =0;
//	ADIE =0;
	temp =ADRESH;
	temp =( temp<<8 )| ADRESL;
//	temp &= 0xfffe;//LSB
	return temp;
}

void lcd_valueout(unsigned int temp){	//[TvXΉ
	if (temp<1000) {
		lcd_putch(' ');
	}else	lcd_putch(((temp /1000)%10)+0x30);//1000
	if (temp<100) {
		lcd_putch(' ');
	}else	lcd_putch(((temp / 100)%10)+0x30);//100
	if (temp<10) {
		lcd_putch(' ');
	}else	lcd_putch(((temp /  10)%10)+0x30);//10
	lcd_putch(((temp      )%10)+0x30);//1
	return;
}

void sci_valueout(unsigned int temp){
	sci_PutByte(((temp /1000)%10)+0x30);//1000
	sci_PutByte(((temp / 100)%10)+0x30);//100
	sci_PutByte(((temp /  10)%10)+0x30);//10
	sci_PutByte(((temp      )%10)+0x30);//1
	return;
}
void delay_10ms(unsigned int temp){	//10ms multiple x times
	while(temp--){
		__delay_ms(10);
	}	
}

void interrupt handler( void ){	//interrupt handle
	unsigned char temp;
	unsigned char data;
	//Handle RS232 Recieve interrupt
	if( RCIF == 1 ){
		//recive to data
		data = sci_GetByte();
		//set to FIFO
		RCIF = 0; //clear RS232 Recieve interrupt bit
		if(FIFO_datasize<MAX_BUFF){
			Buffer_FIFO[FIFO_write_pointer]=data;
			FIFO_write_pointer=(FIFO_write_pointer+1) & (MAX_BUFF-1);//ring buff 
			FIFO_datasize++;
			FIFO_afterglow_indicate_count=20;
		}
	}
}

//[U[LN^[tHgLCDCG RAM֓]
lcd_CG_init(void){
	unsigned char i,j,temp;
	lcd_write(0x40);				//CG RAM[h 0ԒnZbg
	temp = EEPROM_CGRAM_Start_address;	//CG RAM start address set
	for (i=0;i<8;i++){
		for (j=0;j<8;j++){
			lcd_cgwrite(EEPROM_READ(temp++));//CG RAMLN^Zbg(8ij
		}
	}
	lcd_write(0x80);				//DD RAM [hZbg
	lcd_goto(0x00);					//LCD̕\ʒuz[փZbg
	return;
}
//------------------main
void main(void){
//gpϐ̐錾
	unsigned char temp,i,j,bar,select_ch=0,counter=0,key_release=0;
	unsigned int timer,adc_value,adc_data[7];
	unsigned int tank_adc_value16x=0;	//ADl̕ω16{lۑ
	
//NbNݒ
	OSCCON = 0b01110000;	//8MHz,__configłFOSCݒ
		
//e|[g̏
//		  	  76543210
	PORTA = 0b00000000;		//|[gA
	PORTB = 0b00000000;		//|[gB
	TRISA = 0b00111111; 	//|[gA̕ݒ
	TRISB = 0b11100100; 	//|[gB̕ݒ

	ANSEL = 0b01111111;		//AiO|[gŎgp
	CMCON = 0x07;			//|[gAfW^I/OƂĎgp
	RBPU  = 0;				//|[gBvAbvݒ
	PEIE = 1;				//ӊ荞݋
	GIE  = 1;				//荞݋
	

	adinit();//adRo[^
	
//ԑ҂id肷܂0.2b҂)
	delay_10ms(20);		//0.01bx20 = 0.2b^C}[ 


//LCD
	lcd_init();			//LCD
	lcd_clear();		//LCD\NAɂ
	lcd_write(0x0c);	//J[\\Ȃ
	lcd_CG_init();		//CG RAM̓]

//ʐM
	sci_Init( USART_SPEED,SCI_EIGHT);// Baud set and Bit set
	CREN =0;	//M~
	while (RCIF ==1){
		temp = sci_GetByte();		//ǂ
	}
	//sci check
	if (sci_CheckOERR()) { //sci error
		while (!TRMT);
		CREN=1;
	}
//NWbgM
	sci_puts("***7ch AD convert start***\n\r");
	sci_puts("(C)2008 air_variable\n\r");
	VERSION_SEND;
//	sci_puts("command usage\n\r");
//	sci_puts("0to7:individial channel select\n\r");
//	sci_puts("a:output all channel\n\r");
//	sci_puts("c:clear console\n\r");
	
	RCIE = 1;	//ʐM荞݋	
	CREN = 1;	//MJn
	
//\Jn@ivɌJԂ------------------------------

	while(1){		

//ADl擾
		for (i=0;i<7;i++){	//SẴ`lADl擾
			adc_data[i]=  adconv(i);
		}	

//MR}h
		if (FIFO_datasize !=0){	// FIFO𒲍āAf[^݂ǂ݂ɍs
			temp = Buffer_FIFO[FIFO_read_pointer];
			FIFO_read_pointer=(FIFO_read_pointer+1)&(MAX_BUFF-1);//ring buff
			FIFO_datasize--;

			if ((temp>='0')&&(temp<'7')){ //Y`l̑M
					sci_valueout( adc_data[temp & 0x07] );
					NEWLINE;
			}else{
				switch (temp){
					case 'a':	//SẴ`lf[^̑M
					case 'A':
						for(i=0;i<6;i++){
							sci_valueout(adc_data[i]);
							sci_PutByte(',');
						}
						sci_valueout(adc_data[6]);
						NEWLINE;
						break;
					case 'c':	//R\[NA
					case 'C':	
						sci_PutByte(12);
						break;
					default:	//	
						sci_PutByte( temp );
				}
			}
		}		

//LCD\i	
		lcd_goto(0x00);	
//		adc_value = adconv(select_ch);		//ADRo[^l擾@fobOp
		adc_value = adc_data[select_ch];		//[^l擾
		tank_adc_value16x = (adc_value * 16 + tank_adc_value16x * (ADC_response-1))/ADC_response;//ω16{
		adc_value = tank_adc_value16x /16;// 1/16
		if ( (tank_adc_value16x % 16 ) > 7) adc_value++;		//ľܓ
		lcd_puts("CH");
//		lcd_valueout(adconv(select_ch));//fobOp
		lcd_putch(0x30 + select_ch);
		lcd_putch(':');
		lcd_valueout(adc_value);

//Ot쐬
		lcd_goto(0x08);
		if (FIFO_afterglow_indicate_count >0){//ʐMMobt@̃CWP[^[}[N\
			lcd_putch('*');
			FIFO_afterglow_indicate_count--;
		}else{
			lcd_putch(	progress_mark[(counter++>>5)%2] );//vOXCWP[^[\
		}
		for (i=0;i<7;i++){
			lcd_putch( minibar_mark[(adc_data[i]+171)/ 172]);
		}	

//LCDiɃo[Ot
		bar = (unsigned char)(adc_value/16) 		;//o[̑傫vZ
		lcd_goto(0x40);				//LCD̕\ʒu2iڂ(SC1602BS̏ꍇ)

		temp = bar /4;				//64iK16iKɂ
		i=temp;
		while(i--){
			lcd_putch( bar_mark[3] );	//傫o[\
		}
		if ((adc_value>0)&&(adc_value<16)) {//1`15̓o[\
			lcd_putch( bar_mark[1] );
		}	else lcd_putch( bar_mark[bar%4] );	//ׂo[vZĕ\
		i=16-temp;			//㔼LN^
		while(--i){
			lcd_putch(' ');
		}	
	
//{^`FbN	
		if (( !PSW ) && (key_release==0)) {		//{^`FbN
			select_ch++;
			key_release++;
		}	
		if(( !PSW ) && (key_release>0)){		//{^s[g
			key_release++;
			if (key_release>100){
				key_release =100;
				select_ch++;
				delay_10ms(20);
			}
		}		
		if (select_ch >6){
			select_ch=0;
		}

		if ( ( PSW ) && (key_release>0)){		//{^ꂽ̃`FbN
			key_release=0;
		}
	}
}		
